@@ -30,6 +30,7 @@ WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include <cstdlib>
 #include <climits>
 
+#include <algorithm>
 #include <map>
 #include <set>
 #include <string>
@@ -159,6 +160,22 @@ static int setset_parse_set(PyObject* so, set<int>* s) {
   return 0;
 }
 
+static vector<int> intersection(const map<string, vector<int> >& m,
+                                const string& key1, const string& key2) {
+  map<string, vector<int> >::const_iterator in_i = m.find(key1);
+  map<string, vector<int> >::const_iterator ex_i = m.find(key2);
+  vector<int> in_v = in_i != m.end() ? in_i->second : vector<int>();
+  vector<int> ex_v = ex_i != m.end() ? ex_i->second : vector<int>();
+  std::sort(in_v.begin(), in_v.end());
+  std::sort(ex_v.begin(), ex_v.end());
+  vector<int> v(std::max(in_v.size(), ex_v.size()));
+  vector<int>::const_iterator end
+      = std::set_intersection(in_v.begin(), in_v.end(), ex_v.begin(), ex_v.end(),
+                              v.begin());
+  v.resize(end - v.begin());
+  return v;
+}
+
 static int setset_parse_map(PyObject* dict_obj, map<string, vector<int> >* m) {
   assert(m != NULL);
   PyObject* key_obj;
@@ -170,6 +187,10 @@ static int setset_parse_map(PyObject* dict_obj, map<string, vector<int> >* m) {
       return -1;
     }
     string key = PyString_AsString(key_obj);
+    if (key != "include" && key != "exclude") {
+      PyErr_SetString(PyExc_TypeError, "invalid dict key");
+      return -1;
+    }
     PyObject* i = PyObject_GetIter(lo);
     if (i == NULL) return -1;
     vector<int> v;
@@ -186,6 +207,10 @@ static int setset_parse_map(PyObject* dict_obj, map<string, vector<int> >* m) {
     Py_DECREF(i);
     (*m)[key] = v;
   }
+  if (!intersection(*m, "include", "exclude").empty()) {
+    PyErr_SetString(PyExc_TypeError, "inconsistent constraints");
+    return -1;
+  }
   return 0;
 }
 
@@ -741,18 +766,23 @@ static PyObject* setset_load(PySetsetObject* self, PyObject* obj) {
   FILE* fp = PyFile_AsFile(obj);
   PyFileObject* file = reinterpret_cast<PyFileObject*>(obj);
   PyFile_IncUseCount(file);
+  PySetsetObject* ret;
   Py_BEGIN_ALLOW_THREADS;
-  self->ss->load(fp);
+  ret = reinterpret_cast<PySetsetObject*>(
+      PySetset_Type.tp_alloc(&PySetset_Type, 0));
+  ret->ss = new setset(setset::load(fp));
   Py_END_ALLOW_THREADS;
   PyFile_DecUseCount(file);
-  Py_RETURN_NONE;
+  return reinterpret_cast<PyObject*>(ret);
 }
 
 static PyObject* setset_loads(PySetsetObject* self, PyObject* obj) {
   CHECK_OR_ERROR(obj, PyString_Check, "str", NULL);
   stringstream sstr(PyString_AsString(obj));
-  self->ss->load(sstr);
-  Py_RETURN_NONE;
+  PySetsetObject* ret = reinterpret_cast<PySetsetObject*>(
+      PySetset_Type.tp_alloc(&PySetset_Type, 0));
+  ret->ss = new setset(setset::load(sstr));
+  return reinterpret_cast<PyObject*>(ret);
 }
 
 static PyObject* setset_enum(PySetsetObject* self, PyObject* obj) {
@@ -872,8 +902,6 @@ static PyMethodDef setset_methods[] = {
   {"choice", reinterpret_cast<PyCFunction>(setset_choice), METH_NOARGS, ""},
   {"dump", reinterpret_cast<PyCFunction>(setset_dump), METH_O, ""},
   {"dumps", reinterpret_cast<PyCFunction>(setset_dumps), METH_NOARGS, ""},
-  {"load", reinterpret_cast<PyCFunction>(setset_load), METH_O, ""},
-  {"loads", reinterpret_cast<PyCFunction>(setset_loads), METH_O, ""},
   {"_enum", reinterpret_cast<PyCFunction>(setset_enum), METH_O, ""},
   {"_enums", reinterpret_cast<PyCFunction>(setset_enums), METH_NOARGS, ""},
   {NULL}  /* Sentinel */
@@ -1121,6 +1149,8 @@ static PyObject* graphset_graphs(PyObject*, PyObject* args, PyObject* kwds) {
 }
 
 static PyMethodDef module_methods[] = {
+  {"load", reinterpret_cast<PyCFunction>(setset_load), METH_O, ""},
+  {"loads", reinterpret_cast<PyCFunction>(setset_loads), METH_O, ""},
   {"_elem_limit", reinterpret_cast<PyCFunction>(setset_elem_limit), METH_NOARGS, ""},
   {"_num_elems", setset_num_elems, METH_VARARGS, ""},
   {"_graphs", reinterpret_cast<PyCFunction>(graphset_graphs), METH_VARARGS | METH_KEYWORDS, ""},
